%%% Filename : ac_axiom2rule.pl
%%% Date     : 22.03.2012
%%% Contents : main method to create a rule from an axiom
%%% Author   : Lara Katharina Spendier <lara@logic.at>


%% is_rule(+Axiom, +Class)
%%		creates the analytic rule out of the axiom provided by the user
%%		special case N2: several axioms might be concatenated by an &
%%
%% 		Axiom ... Axiom provided by the user
%%		Class ... Class the axiom is in
axiom2rule(Axiom, C) :-
		member(C, [p(1), n(1), n(2)]),
		is_several_axioms(Axiom, LAxioms),
		reverseList(LAxioms, [], LAxioms1),
		is_rules(LAxioms1, C, Analytic, 0),
		texOut(Axiom, Analytic, C).

axiom2rule(Axiom, C) :-
		member(C, [p(2), p(3)]),
		is_hypersequent(Axiom, Hyperseq),
		reverseList(Hyperseq, [], Hyperseq1),
		is_several_hyps([Hyperseq1],LHyperseq),
		reverseList(LHyperseq,[],LHyperseq1),
		is_rules(LHyperseq1, C, Analytic,0),
		texOut(Axiom,Analytic,C).

axiom2rule(_, C) :-
		\+ member(C, [p(1), p(2), p(3), n(1), n(2)]),
		writef('\nThe algorithm does not work for this class yet.'),
		texOut(_,_,C).

%% is_several_axioms(+Axiom, -Axioms)
%%		gets a list of the &-concatenated axioms which can occur on level N2
%% 		Axiom  ... Axiom provided by the user
%%		Axioms ... List of axioms
is_several_axioms(Ax1 & Ax2, HS1) :-
	is_several_axioms(Ax1, HS0),
	append([Ax2], HS0, HS1).

is_several_axioms(Ax, HS) :-
	atomic(Ax),
	append([Ax], [], HS).

is_several_axioms(Ax, HS) :-
	compound(Ax),
	append([Ax], [], HS).

%% is_rule(+Axioms, +Class, -Analytic, +SC)
%%		creates the analytic rules out of the axiom provided by the user
%% 		Axioms   ... List of N2-&-concatenated axioms (based on the axiom provided by the user)
%%		Class    ... Class the axiom is in
%%		Analytic ... List of analytic rules according to Axioms
%%		SC ... flag whether the "simplified" rule should be computes (due to standard
%%			   completeness check, = 1) or not ( = 0)
is_rules([], _, [],_).
is_rules([H|T], Class, Analytic, SC) :-
		athsr(H, Class, Hyperstructural),
		htar(Hyperstructural, Analytic1,SC),
		( SC = 0 ->
		  writef('\n\nEquivalent Analytic Rule: \n')
		; writef('\n\nEquivalent Analytic Rule (in presence of the axiom (a -> b) v (b -> a)): \n')
		), 
		printrule(Analytic1, Class),
		is_rules(T, Class, Analytic2,SC),
		append([Analytic1], Analytic2, Analytic).

%% is_standardcomplete(+Axiom, +Class)
%% 		Axiom ... Axiom provided by the user
%%		Class ... Class the axiom is in
is_standardcomplete(_, C) :-
		\+ member(C, [p(1), p(2), p(3), n(1), n(2)]),
		writef('\nWe cannot yet determine standard completeness for a rule in this class.'),
		texOutSC(_, _, _, 100, _).

is_standardcomplete(Axiom, C) :-
		member(C, [p(1), n(1), n(2)]),
		is_several_axioms(Axiom, LAxioms),
		reverseList(LAxioms, [], LAxioms1),
		is_rules(LAxioms1, C, Analytic,1),
		writef('\n\n The logic obtained by extending MTL with the axiom is standard complete.'),
		texOutSC(Axiom, Analytic, C, 1, []).

is_standardcomplete(Axiom, C) :-
		member(C, [p(2), p(3)]),
		is_hypersequent(Axiom, Hyperseq),
		reverseList(Hyperseq, [], Hyperseq1),
		is_several_hyps([Hyperseq1],LHyperseq),
		reverseList(LHyperseq,[],LHyperseq1),
		is_sc_rules(LHyperseq1, C, Analytic, Convergent, ConvPrem),
		%nl,writef('Analytic: %q',[Analytic]),
		( (is_semianchlist(Analytic)) ->
		  (SemiAnchored = 1,
		   write('\n The logic obtained by extending MTL with the axiom is standard complete.'));
           (SemiAnchored = 0,
        	write('\n The generated rule(s) is(are) not semi-anchored.'))
        ),  
        ( (Convergent = 1) ->
  	      write('\n The generated rule(s) is(are) convergent.')
		; write('\n The generated rule(s) is(are) not convergent.')
		),
		texOutSCA(Axiom, Analytic, C, Convergent, ConvPrem, SemiAnchored).

is_sc_rules([], _, [], 1, [[],[]]).
is_sc_rules([H|T], Class, Analytic, Convergent, [Prem,Conv]) :-
		athsr(H, Class, Hyperstructural),
		htar(Hyperstructural, Analytic1,1),
		writef('\n\nEquivalent Analytic Rule (in presence of the axiom (a -> b) v (b -> a)): \n'),
		printrule(Analytic1, Class),
		is_convergent(Analytic1, Convergent1, [Prem1,Conv1]),
		is_sc_rules(T, Class, Analytic2, Convergent2, [Prem2,Conv2]),
		append([Analytic1], Analytic2, Analytic),
		append(Prem1,Prem2,Prem),
		append(Conv1, Conv2, Conv),
		( (Convergent1 = 0; Convergent2 = 0) ->
		  Convergent = 0
		; Convergent = 1).


