%%% Filename : ac_checkclass.pl
%%% Date     : 23.02.2011
%%% Contents : gets the class of the substructural hierarchy of an axiom
%%% Author   : Lara Katharina Spendier <lara@logic.at>

%% is_pos_class/is_neg_class(+Axiom, +Class, -Class)
%%		gets the positive/negative class of an axiom
is_pos_class(A, P0, P0) :-
	is_pos_axiom(A, P0).
	
is_pos_class(A, P0, P2) :-
	\+ is_pos_axiom(A, P0),
	P1 is P0+1,
	is_pos_class(A, P1, P2).
	
is_neg_class(A, N0, N0) :-
	is_neg_axiom(A, N0).
	
is_neg_class(A, N0, N2) :-
	\+ is_neg_axiom(A, N0),
	N1 is N0+1,
	is_neg_class(A, N1, N2).
			
%% is_in_class(+PositiveClass, +NegativeClass, -Class)
%%		gets the class an axiom is in, based on the level of the positive/negative class
is_in_class(P, N, C) :-
	( P < N -> C = p(P) 
	; C = n(N) ).

%% is_atom(+Axiom)
%%		Every number and every Prolog atom is an atomic formula, except 0, 1, top, bot
is_atom(A) :-
        atomic(A),
        \+ member(A,[0,1,bot,top]).

%% is_pos_axiom(+Axiom, +N)
%%		checks whether the axiom A is in the positive class on level N
%% hierarchy: P_0 :- A; P_{n+1} :- N_n | P_{n+1} * P_{n+1} | P_{n+1} v P_{n+1} | 1 | bot

is_pos_axiom(A, 1) :-     	% P_0 :- A
	is_atom(A).
is_pos_axiom(1, N) :- 	    % P_{n+1} :- 1
	N > 0.
is_pos_axiom(bot, N) :-    	% P_{n+1} :- \Bottom
	N > 0.
is_pos_axiom(A, N1) :-    	% P_{n+1} :- N_n
	N1 > 0,
    N is N1-1,
    is_neg_axiom(A, N).
is_pos_axiom(A v B, N) :- 	% P_{n+1} :- P_{n+1} v P_{n+1}
	N > 0,
    is_pos_axiom(A, N),
    is_pos_axiom(B, N).
is_pos_axiom(A * B, N) :- 	% P_{n+1} :- P_{n+1} * P_{n+1}
	N > 0,
	is_pos_axiom(A, N),
	is_pos_axiom(B, N).

%% is_neg_axiom(+Axiom, +N)
%%		checks whether the axiom A is in the positive class on level N
%% hierarchy: N_0 :- A; N_{n+1} :- P_n | P_{n+1} -> N_{n+1} | N_{n+1} & N_{n+1} | 0 | top

is_neg_axiom(A, 1) :-      	% N_0 :- A
    is_atom(A).
is_neg_axiom(0, N) :-       % N_{n+1} :- 0
	N > 0.
is_neg_axiom(top, N) :-     % N_{n+1} :- T
	N > 0.
is_neg_axiom(A, N1) :-     	% N_{n+1} :- P_n
	N1 > 0,
    N is N1-1,
    is_pos_axiom(A, N).
is_neg_axiom(-A, N) :-      % -A = A -> 0 = N_{n+1}
    N > 0,
    is_neg_axiom(A -> 0, N).
is_neg_axiom(A -> B, N) :- 	% N_{n+1} :- P_{n+1} -> N_{n+1}
	N > 0,
    is_pos_axiom(A, N),
    is_neg_axiom(B, N).
is_neg_axiom(A & B, N) :- 	% N_{n+1} :- N_{n+1} & N_{n+1}
	N > 0,
	is_neg_axiom(A, N),
	is_neg_axiom(B, N).

%% is_in_NC(+Axiom)
%% 	checks whether the axiom A is of the shape of the axiom class NC
%%	as we do not work with noncommutative hypersequents --> not needed

%is_in_NC(A) :-
%	is_atom(A).
%
%is_in_NC(D) :-
%	writef('\ntry D'),
%	is_neg_axiom(D, 2),
%	writef('\nisnegclass D, %p', [D]).
%
%is_in_NC( G -> (D v (A))) :-
%	writef('\ntry G -> (D v (A))'),
%	is_neg_class(G -> D, 0, N),
%	N < 3,
%	D \= (_ -> _),
%	D \= (_ & _),
%	writef('\nisnegclass G -> (D v (A)), %p => %p OR %p', [G, D, A]),
%	is_in_NC(A).
%
%is_in_NC(D v (A)) :-
%	writef('\ntry D v A'),
%	is_neg_class(top -> D, 0, N),
%	N < 3,
%	D \= (_ -> _),
%	D \= (_ & _),
%	writef('\nisnegclass top -> D, %p OR %p', [D, A]),
%	is_in_NC(A).
