%%% Filename : ac_grammar.pl
%%% Date     : 16.03.2012
%%% Contents : DCG grammar for tex-output
%%% Author   : Lara Katharina Spendier <lara@logic.at>

bs        	  --> [92].
eol 	  	  --> [13,10].

%% Definitions for the style-file

texNewAxiom(C)		--> bs, "def", bs, "NewAxiom", "{$", C, "$}", eol.
texNewAxiomNotMath(C)	--> bs, "def", bs, "NewAxiomNotMath", "{", C, "}", eol.
texNewRuleStart   	--> bs, "def", bs, "NewRule", "{".
texNewRuleEnd	  	--> "}", eol.
texAxiomClass(C)  	--> bs, "def", bs, "AxiomClass", "{$", bs, "mathcal", C, "$}", eol.
texNameBaseCalculus(C) 	--> bs, "def", bs, "NameBaseCalculus{",C,"}",eol.
texNameBaseCalculusP(C)	--> bs, "def", bs, "NameBaseCalculusP{",C,"}",eol.
texBaseCalculus(C) 	--> bs, "def", bs, "BaseCalculus{",bs,C,"}",eol.
texEquivalences(C) 	--> bs, "def", bs, "Equivalences{",bs,C,"}",eol.
texWrittenBC(C)	   	--> bs, "def", bs, "WrittenBC{",C,"}",eol.
texFullLambek(C)   	--> bs, "def", bs, "FullLambek{",bs,C,"}",eol.
texTVPos(C)		--> bs, "def", bs, "TVPos{",bs,C,"}",eol.
texTVNeg(C)		--> bs, "def", bs, "TVNeg{",bs,C,"}",eol.
texTVText(C)		--> bs, "def", bs, "TVText{",bs,C,"}",eol.
texRule(C)		--> bs, "def", bs, "WRule{",C,"}",eol.
texRuleVerb(C)		--> bs, "def", bs, "WRuleV{",C,"}",eol.
texConvergentRule(C)	--> bs, "def", bs, "ConvergentRule{",C,"}",eol.
texConvergentRule(C,D,E)	--> bs, "def", bs, "ConvergentRule{",C,D,E,"}",eol.
texStandardComplete(C)	--> bs, "def", bs, "StandardComplete{",bs,C,"}",eol.
texAbstract(C)		--> bs, "def", bs, "Abstract{",bs,C,"}",eol.
texTitle(C)		--> bs, "def", bs, "Title{",bs,C,"}",eol.
texSectionTitle(C)	--> bs, "def", bs, "SectionTitle{",bs,C,"}",eol.
texNotConvergent(C) --> bs, "def", bs, "NotConvergent{",C,"}",eol.
texConvOrSA(C) --> bs, "def", bs, "ConvOrSA{",C,"}",eol.
texConvOrSAText(C) --> bs, "def", bs, "ConvOrSAText{",bs,C,"}",eol.

%% tex-rewriting of the pivot premises
pivot2tex([])		--> "",eol.
pivot2tex([[0, H]|[]])	--> bs,"item The following premises are a ",bs,"textbf{0-pivot}:",bs,bs,"$",prem2tex(H),"$".
pivot2tex([[N, H]|[]])	--> bs,"item There exists a ",bs,"textbf{",nr2tex(N),"-pivot} for the following premises:",bs,bs,"$",prem2tex(H),"$".
pivot2tex([[0, H]|T])	--> bs,"item The following premises are a ",bs,"textbf{0-pivot}:",bs,bs,"$",prem2tex(H),"$",pivot2tex(T).
pivot2tex([[N, H]|T])	--> bs,"item There exists a ",bs,"textbf{",nr2tex(N),"-pivot} for the following premises:",bs,bs,"$",prem2tex(H),"$",pivot2tex(T).
prem2tex([[[A|A1],S]|T])--> "G",bs,"hh",prem2tex([A|A1]),bs,"Ra",prem2tex(S),bs,"quad ",prem2tex(T).
prem2tex('G'+N)   	--> bs,"Gamma_{",nr2tex(N),"}".
prem2tex('D'+N)   	--> bs,"Delta_{",nr2tex(N),"}".
prem2tex('P'+N)   	--> bs,"Pi_{",nr2tex(N),"}".
prem2tex('')      	--> "".
prem2tex([])	  	--> "".
prem2tex([H|[]])  	--> prem2tex(H).
prem2tex([H|T])   	--> prem2tex(H), ",", prem2tex(T).

%% tex-rewriting of the (hyper)sequent rules with >= 0 premises

texNewAxC(C)      --> bs, "AxiomC{$",C,"$}",eol.
texNewUnC(C)	  --> bs, "UnaryInfC{$",C,"$}",eol.
texNoLine	  --> bs, "noLine", eol.

texNewUnCon(C)    --> bs, "UnaryInfC{$",C,"$}",bs,"DisplayProof",bs,"hspace*{0.3cm}",eol.
texNewBinCon(C)   --> bs, "BinaryInfC{$",C,"$}",bs,"DisplayProof",bs,"hspace*{0.3cm}",eol.
texNewTriCon(C)   --> bs, "TrinaryInfC{$",C,"$}",bs,"DisplayProof",bs,"hspace*{0.3cm}",eol.

rule2tex([[H|T],C])  --> rule2tex([H|T]),bs,"Ra",rule2tex(C).
rule2tex('G'+N)   --> bs,"Gamma_{",nr2tex(N),"}".
rule2tex('D'+N)   --> bs,"Delta_{",nr2tex(N),"}".
rule2tex('P'+N)   --> bs,"Pi_{",nr2tex(N),"}".
rule2tex('')      --> "".
rule2tex([])	  --> "".
rule2tex([H|[]])  --> rule2tex(H).
rule2tex([H|T])   --> rule2tex(H), ",", rule2tex(T).

nr2tex(N)	  --> {number_codes(N, Cs)}, Cs.

%%
%% axiom2tex:   rewrites the axiom provided by the user to tex
%% axiom2texP1: takes care of bracketing for formulas X &/v/* Y
%% axiom2texP2: takes care of bracketing for formulas X -> Y
%%

axiom2texP1(-X)   --> " ", bs, "lnot ", axiom2texP1(X).
axiom2texP1(X & Y)--> "(", axiom2texP2(X), " ", bs, "land ", axiom2texP2(Y), ")".
axiom2texP1(X v Y)--> "(", axiom2texP3(X), " ", bs, "lor ", axiom2texP3(Y), ")".
axiom2texP1(X * Y)--> "(", axiom2texP4(X), " ", bs, "cdot ", axiom2texP4(Y), ")".
axiom2texP1(X -> Y)--> "(", axiom2texP5(X), " ", bs, "ra ", axiom2texP5(Y), ")".
axiom2texP1(X)	  --> {atomic(X);member(X, [0,1,bot,top])}, axiom2tex(X).

axiom2texP2(-X)   --> " ", bs, "lnot ", axiom2texP1(X).
axiom2texP2(X & Y)--> axiom2texP2(X), " ", bs, "land ", axiom2texP2(Y).
axiom2texP2(X v Y)--> "(", axiom2texP3(X), " ", bs, "lor ", axiom2texP3(Y), ")".
axiom2texP2(X * Y)--> "(", axiom2texP4(X), " ", bs, "cdot ", axiom2texP4(Y), ")".
axiom2texP2(X -> Y)--> "(", axiom2texP5(X), " ", bs, "ra ", axiom2texP5(Y), ")".
axiom2texP2(X)	  --> {atomic(X);member(X, [0,1,bot,top])}, axiom2tex(X).

axiom2texP3(-X)   --> " ", bs, "lnot ", axiom2texP1(X).
axiom2texP3(X & Y)--> "(", axiom2texP2(X), " ", bs, "land ", axiom2texP2(Y), ")".
axiom2texP3(X v Y)--> axiom2texP3(X), " ", bs, "lor ", axiom2texP3(Y).
axiom2texP3(X * Y)--> "(", axiom2texP4(X), " ", bs, "cdot ", axiom2texP4(Y), ")".
axiom2texP3(X -> Y)--> "(", axiom2texP5(X), " ", bs, "ra ", axiom2texP5(Y), ")".
axiom2texP3(X)	  --> {atomic(X);member(X, [0,1,bot,top])}, axiom2tex(X).

axiom2texP4(-X)   --> " ", bs, "lnot ", axiom2texP1(X).
axiom2texP4(X & Y)--> "(", axiom2texP2(X), " ", bs, "land ", axiom2texP2(Y), ")".
axiom2texP4(X v Y)--> "(", axiom2texP3(X), " ", bs, "lor ", axiom2texP3(Y), ")".
axiom2texP4(X * Y)--> axiom2texP4(X), " ", bs, "cdot ", axiom2texP4(Y).
axiom2texP4(X -> Y)--> "(", axiom2texP5(X), " ", bs, "ra ", axiom2texP5(Y), ")".
axiom2texP4(X)	  --> {atomic(X);member(X, [0,1,bot,top])}, axiom2tex(X).

axiom2texP5(-X)   --> " ", bs, "lnot ", axiom2texP1(X).
axiom2texP5(X & Y)--> axiom2texP2(X), " ", bs, "land ", axiom2texP2(Y).
axiom2texP5(X v Y)--> axiom2texP3(X), " ", bs, "lor ", axiom2texP3(Y).
axiom2texP5(X * Y)--> axiom2texP4(X), " ", bs, "cdot ", axiom2texP4(Y).
axiom2texP5(X -> Y)--> "(", axiom2texP5(X), " ", bs, "ra ", axiom2texP5(Y), ")".
axiom2texP5(X)	  --> {atomic(X);member(X, [0,1,bot,top])}, axiom2tex(X).

axiom2tex(-X) 	  --> " ", bs, "lnot ", axiom2texP1(X).
axiom2tex(X & Y)  --> axiom2texP2(X), " ", bs, "land ", axiom2texP2(Y).
axiom2tex(X v Y)  --> axiom2texP3(X), " ", bs, "lor ", axiom2texP3(Y).
axiom2tex(X * Y)  --> axiom2texP4(X), " ", bs, "cdot ", axiom2texP4(Y).
axiom2tex(X -> Y) --> axiom2texP5(X), " ", bs, "ra ", axiom2texP5(Y).
axiom2tex(1) 	  --> "1".
axiom2tex(bot) 	  --> bs,"bot".
axiom2tex(top) 	  --> bs,"top".
axiom2tex(0) 	  --> "0".
axiom2tex(a) 	  --> bs,"alpha".
axiom2tex(b) 	  --> bs,"beta".
axiom2tex(c)	  --> bs, "gamma".
axiom2tex(d)	  --> bs, "delta".
axiom2tex(e)	  --> bs, "varepsilon".
axiom2tex(f)	  --> bs, "zeta".
axiom2tex(g)	  --> bs, "eta".
axiom2tex(h)	  --> bs, "theta".
axiom2tex(i)	  --> bs, "iota".
axiom2tex(j)	  --> bs, "xi".
axiom2tex(k)	  --> bs, "kappa".
axiom2tex(l)	  --> bs, "lambda".
axiom2tex(m)	  --> bs, "mu".
axiom2tex(n)	  --> bs, "nu".
axiom2tex(o)	  --> bs, "omicron".
axiom2tex(p)	  --> bs, "pi".
axiom2tex(q)	  --> bs, "varpi".
axiom2tex(r)	  --> bs, "rho".
axiom2tex(s)	  --> bs, "sigma".
axiom2tex(t)	  --> bs, "tau".
axiom2tex(u)	  --> bs, "upsilon".
axiom2tex(w)	  --> bs, "psi".
axiom2tex(x)	  --> bs, "chi".
axiom2tex(y)	  --> bs, "psi".
axiom2tex(z)	  --> bs, "omega".

print_phrase(V) :-
   phrase(V, S),
   name(A, S),
   write(A).


