%%% Filename : ac_kernl.pl
%%% Date     : 23.02.2011
%%% Contents : main method to start the program
%%% Author   : Lara Katharina Spendier <lara@logic.at>

:- use_module(library(time)).
:- ['ac_operators.pl'].
:- ['ac_listfunctions.pl'].
:- ['ac_checkclass.pl'].
:- ['ac_hyperstructural.pl'].
:- ['ac_analytic.pl'].
:- ['ac_printcmd.pl'].
:- ['ac_grammar.pl'].
:- ['ac_tex.pl'].
:- ['ac_axiom2rule.pl'].
:- ['ac_convergent.pl'].
:- ['ac_semianc.pl'].

compute :-
		catch(
		  read(Axiom),
		  Exception,
		  handle_read(Exception)
		),
  		(\+ axiom2tex(Axiom,_,[]) ->
		handle("1")
		; 
		is_pos_class(Axiom, 0, P),
		is_neg_class(Axiom, 0, N),
		is_in_class(P, N, C),
		writef('\nThis axiom is in the class: %q\n', [C]),
        catch(
                call_with_time_limit(120, axiom2rule(Axiom, C)),
                Exception,                     
                handle_timeout(Exception))
		).

computeweb(Axiom) :-
		(\+ axiom2tex(Axiom,_,[]) ->
		handle_read("1")
		; 
		is_pos_class(Axiom, 0, P),
		is_neg_class(Axiom, 0, N),
		is_in_class(P, N, C),
		writef('\nThis axiom is in the class: %q\n', [C]),
		catch(
                call_with_time_limit(4, axiom2rule(Axiom, C)),
                Exception,                     
                handle_timeout(Exception))
		),
		halt.

computesc :-
		catch(
		  read(Axiom),
		  Exception,
		  handle_read(Exception)
		),
		(\+ axiom2tex(Axiom,_,[]) ->
		handle("1")
		; is_pos_class(Axiom, 0, P),
		  is_neg_class(Axiom, 0, N),
		  is_in_class(P, N, C),
		  writef('\nThis axiom is in the class: %q\n', [C]),
		  catch(
		  call_with_time_limit(120, is_standardcomplete(Axiom,C)),
		  Exception,
		  handle_timeout(Exception))
		).

computewebsc(Axiom) :-
		(\+ axiom2tex(Axiom,_,[]) ->
		handle_read("1")
		; is_pos_class(Axiom, 0, P),
		  is_neg_class(Axiom, 0, N),
		  is_in_class(P, N, C),
		  writef('\nThis axiom is in the class: %q\n', [C]),
		  catch(
		  call_with_time_limit(4, is_standardcomplete(Axiom,C)),
		  Exception,
		  handle_timeout(Exception))
		),
		halt.

handle_read(_) :-	
		texOut(_,_,100),
		texOutSC(_,_,_,100,_),
		writef('\n\nThe axiom you provided was not computable.\n'),
		halt.

handle_timeout(_) :-	
		texOut(_,_,100),
		texOutSC(_,_,_,100,_),
		writef('\n\nThe computation of the axiom timed out.\n'),
		halt.
		
handle(_) :-
        texOut(_,_,100),
		texOutSC(_,_,_,100,_),
		writef('\n\nThe axiom you provided was not computable.'),
		halt.


%%% TESTS
%?- is_convergent([[[['G'+1, 'G'+1], 'P'+1]], [[['G'+1], 'P'+1]]], 1, _).
%?- is_convergent([[[['G'+2, 'G'+1], 'P'+1]], [[['G'+1], 'P'+1]]], 0, _).
%?- is_convergent([[[['G'+2, 'G'+1], 'P'+1]], [[['G'+1, 'G'+2], 'P'+1]]], 1, _).
%?- is_convergent([[[['G'+2, 'G'+1], 'P'+1], [['G'+1, 'G'+1], 'P'+1]], [[['G'+1], 'P'+1]]], 1, _).
%?- is_convergent([[[['G'+1, 'G'+2], 'P'+1], [['G'+3, 'G'+1], 'P'+1], [['G'+1, 'G'+1], 'P'+1]], [[['G'+1], 'P'+1]]], 1, _).
%?- is_convergent([[[['G'+2, 'G'+3], 'P'+1], [['G'+1, 'G'+2], 'P'+1], [['G'+3, 'G'+1], 'P'+1], [['G'+1, 'G'+1], 'P'+1]], [[['G'+1], 'P'+1]]], 1, _).
%?- is_convergent([[[['G'+2, 'G'+3], 'P'+1], [['G'+1, 'G'+2], 'P'+1], [['G'+1, 'G'+1], 'P'+1]], [[['G'+1], 'P'+1]]], 0, _).
%?- is_convergent([[[['G'+1, 'G'+1, 'G'+2], 'P'+1], [['G'+1, 'G'+1, 'G'+1], 'P'+1]], [[['G'+1], 'P'+1]]], 1, _).
%?- is_convergent([[[['G'+1, 'G'+2, 'G'+3], 'P'+1], [['G'+1, 'G'+1, 'G'+2], 'P'+1], [['G'+1, 'G'+1, 'G'+1], 'P'+1]], [[['G'+1], 'P'+1]]], 0, _).
%?- is_convergent([[[['G'+1, 'G'+2, 'G'+3], 'P'+1], [['G'+1, 'G'+1, 'G'+2], 'P'+1], [['G'+1, 'G'+1, 'G'+3], 'P'+1], [['G'+1, 'G'+1, 'G'+1], 'P'+1]], [[['G'+1], 'P'+1]]], 1, _).
%?- is_convergent([[[['G'+2, 'G'+3], 'P'+1],[['G'+1, 'G'+1], 'P'+1]], [[['G'+1], 'P'+1]]], 0, _).
%?- is_convergent([[[['G'+2, 'G'+3], 'P'+1],[['G'+1], 'P'+1]], [[['G'+1], 'P'+1]]], 0, _).
%?- is_convergent([[[['G'+1, 'G'+1], 'P'+1], [['G'+4], 'P'+2]], [[['G'+1], 'P'+1],[['G'+4], 'P'+2]]], 1, _).
%?- is_convergent([[[['G'+1, 'G'+1], 'P'+1], [['G'+4], 'P'+2], [['G'+3], 'P'+1]], [[['G'+1], 'P'+1],[['G'+4], 'P'+2]]], 0, _).
%?- is_convergent([[[['G'+1, 'G'+1], 'P'+1], [['G'+4], 'P'+2], [['G'+3], 'P'+1], [['G'+5], 'P'+1], [['G'+6], 'P'+2]], [[['G'+1], 'P'+1],[['G'+4], 'P'+2]]], 0, _).
%?- is_convergent([[[['G'+4], 'P'+2], [['G'+3], 'P'+1], [['G'+5], 'P'+1], [['G'+6], 'P'+2], [['G'+1, 'G'+1], 'P'+1]], [[['G'+1], 'P'+1],[['G'+4], 'P'+2]]], 0, _).
%?- is_convergent([[[['G'+1, 'G'+1], 'P'+1], [['G'+4, 'G'+4], 'P'+2], [['G'+3, 'G'+4], 'P'+1]], [[['G'+1], 'P'+1],[['G'+4], 'P'+2]]], 0, _).
%?- is_convergent([[[['G'+1, 'G'+1], 'P'+1], [['G'+4, 'G'+4], 'P'+2], [['G'+3, 'G'+4], 'P'+2], [['G'+3, 'G'+5], 'P'+2]], [[['G'+1], 'P'+1],[['G'+4], 'P'+2]]], 0, _).
%?- is_convergent([[[['G'+1, 'G'+1], 'P'+1], [['G'+4, 'G'+4], 'P'+2], [['G'+3, 'G'+4], 'P'+2], [['G'+4, 'G'+5], 'P'+2]], [[['G'+1], 'P'+1],[['G'+4], 'P'+2]]], 1, _).
%?- is_convergent([[[['G'+1, 'G'+1], 'P'+1], [['G'+4, 'G'+4], 'P'+2], [['G'+3, 'G'+4], 'P'+2], [['G'+4, 'G'+5], 'P'+2], [['G'+3, 'G'+5], 'P'+2]], [[['G'+1], 'P'+1],[['G'+4], 'P'+2]]], 1, _).
%?- is_convergent([[[['G'+1, 'G'+1], 'P'+1], [['G'+4, 'G'+4], 'P'+2], [['G'+3, 'G'+4], 'P'+2], [['G'+3, 'G'+5], 'P'+2]], [[['G'+4], 'P'+2],[['G'+1], 'P'+1]]], 0, _).
%?- is_convergent([[[['G'+3, 'G'+1], 'P'+1], [['G'+1, 'G'+1], 'P'+1], [['G'+2, 'G'+3], 'P'+1]], [[['G'+1], 'P'+1]]], 0, _).
%?- is_convergent([[[['G'+1, 'G'+1], 'P'+1], [['G'+2, 'G'+3], 'P'+1]], [[['G'+1], 'P'+1]]], 0, _).
%?- is_convergent([[[['G'+1, 'G'+2, 'G'+3], 'P'+1], [['G'+1, 'G'+1, 'G'+2], 'P'+1], [['G'+1, 'G'+1, 'G'+3], 'P'+1], [['G'+1, 'G'+1, 'G'+1], 'P'+1], [['G'+4, 'G'+2, 'G'+3], 'P'+1],[['G'+1, 'G'+4, 'G'+3], 'P'+1], [['G'+1, 'G'+4, 'G'+2], 'P'+1]], [[['G'+1], 'P'+1]]], 0, _).
%?- is_convergent([[[['G'+1, 'G'+2, 'G'+3], 'P'+1], [['G'+1, 'G'+1, 'G'+2], 'P'+1], [['G'+1, 'G'+1, 'G'+3], 'P'+1], [['G'+1, 'G'+1, 'G'+1], 'P'+1], [['G'+4, 'G'+2, 'G'+3], 'P'+1],[['G'+1, 'G'+4, 'G'+3], 'P'+1], [['G'+1, 'G'+4, 'G'+2], 'P'+1], [['G'+1, 'G'+4, 'G'+1], 'P'+1]], [[['G'+1], 'P'+1]]], 1, _).
%?- is_convergent([[[['G'+1, 'G'+2, 'G'+3], 'P'+1], [['G'+1, 'G'+1, 'G'+2], 'P'+1], [['G'+1, 'G'+1, 'G'+3], 'P'+1], [['G'+1, 'G'+1, 'G'+1], 'P'+1], [['G'+4, 'G'+2, 'G'+3], 'P'+1],[['G'+1, 'G'+4, 'G'+3], 'P'+1], [['G'+1, 'G'+4, 'G'+2], 'P'+1], [['G'+1, 'G'+5, 'G'+1], 'P'+1]], [[['G'+1], 'P'+1]]], 0, _).
%?- is_convergent([[[['G'+1,'G'+1,'G'+1,'G'+1,'D'+1], 'P'+1],[['G'+2,'G'+1,'G'+2,'G'+1,'D'+1], 'P'+1]], [[['G'+2,'G'+3,'G'+4,'G'+5], ''],[['G'+1,'D'+1], 'P'+1]]] , 1, _).
%?- is_convergent([[[['G'+1,'G'+1,'G'+1,'G'+1,'D'+1], 'P'+1],[['G'+2,'G'+1,'G'+2,'G'+1,'D'+1], 'P'+1], [['G'+2,'G'+3,'G'+2,'G'+3,'D'+1], 'P'+1], [['G'+1,'G'+3,'G'+1,'G'+3,'D'+1], 'P'+1], [['G'+4,'G'+3,'G'+4,'G'+3,'D'+1], 'P'+1], [['G'+4,'G'+1,'G'+4,'G'+1,'D'+1], 'P'+1], [['G'+4,'G'+5,'G'+4,'G'+5,'D'+1], 'P'+1], [['G'+2,'G'+5,'G'+2,'G'+5,'D'+1], 'P'+1], [['G'+1,'G'+5,'G'+1,'G'+5,'D'+1], 'P'+1]], [[['G'+2,'G'+3,'G'+4,'G'+5], ''],[['G'+1,'D'+1], 'P'+1]]] , 1, _).
