%%% Filename : ac_listfunctions.pl
%%% Date     : 23.02.2011
%%% Contents : helper methods for list manipulation
%%% Author   : Lara Katharina Spendier <lara@logic.at>

%% append(+Element, +List, -List)
%%		appends an element to a list
append([],L,L).
append([H|T],L2,[H|L3]) :- append(T,L2,L3).

%% remove(+Element, +List, -List)
%%		removes an element from a list
remove(X, L2, L3) :-
	\+ member(X, L2),
	append([], L2, L3).

remove(H, [H|L2], L3) :-
	append(L2, [], L3).

remove(X, [H|L2], L3) :-
	X \= H,
	member(X, L2),
	remove(X, L2, L4),
	append([H], L4, L3).

%% removeAll(+Element, +List, -List)
%%		removes All elements Element from a list
removeAll(X, L2, L3) :-
	\+ member(X, L2),
	append([], L2, L3).

removeAll(H, [H|L2], L3) :-
	removeAll(H, L2, L4),
	append(L4, [], L3).

removeAll(X, [H|L2], L3) :-
	X \= H,
	member(X, L2),
	removeAll(X, L2, L4),
	append([H], L4, L3).

	
%% remove_double(+List, -List)
%%		removes elements that occur twice from a list
remove_double([], L, L).
remove_double([H|T], L2, Result) :-
	\+ member(H, L2),
	append([H], L2, L3),
	remove_double(T, L3, Result).
remove_double([H|T], L2, Result) :-
	member(H, L2),
	remove_double(T, L2, Result).

remove_once([], Remainder, Remainder).
remove_once([Current|Delete], List, X) :-
    select(Current, List, Rest),
    remove_once(Delete, Rest, X).

%%  reverseList
%%		reverses the list
reverseList([],A,A).
reverseList([H|T],A,R) :- reverseList(T,[H|A],R).

%% replacElements(+List, +ReplaceElement, +NewElement, -List)
%%		replaces an element with a new element (which is a list)
replaceElements([], _, _, []).

replaceElements([R|T], R, E, NL1) :-
	replaceElements(T, R, E, NL0),
	append(E, NL0, NL1).
	
replaceElements([H|T], R, E, NL1) :-
	H \= R,
	replaceElements(T, R, E, NL0),
	append([H], NL0, NL1).
	
%% PAOLO	
sm(H,[X|Y], L, W) :- sm(H,Y,[X|L],W).
sm(H,[H|Y], L, W) :- append(L,Y,W).

sms([H|Y],G):-sm(H,G,[],W),sms(Y,W).
sms([],G).

smsleng(X,Y,N):- length(X,N), sms(X,Y). 

sbs([],_).
sbs([H|G],X):- member(H,X), sbs(G,X).

number(H,[H|G],1).
number(X,[H|G],N):- number(X,G,M), N is M+1.

disjoint([A],Y):- not(member(A,Y)).
disjoint([X|Z],Y):- not(member(X,Y)), disjoint(Z,Y). 

substitute([A|B],[C|D],Z,W):- sms([A|B],Z), number(A,Z,N), replacepos(Z,N,C,W1), substitute(B,D,W1,W).

substitute([],[],Z,Z).


replacepos([_|T], 1, X, [X|T]).

replacepos([H|T], I, X, [H|R]):- I > 1, I1 is I-1, replacepos(T, I1, X, R).


replace(A,B,[H],[H]):- A\=H.
replace(A,B,[A],[B]).
replace(A,B,[H|T],[H|V]):- A \= H, replace(A,B,T,V).
replace(A,B, [A|T],[B|V]):-  replace(A,B,T,V).


substituteb([A1|A2],[B1|B2],C,D):- replace(A1,B1,C,D1),  substituteb(A2,B2,D1,D).
substituteb([],[],Z,Z).


removelast([H],[]).
removelast(G,S):- reverseList(G, [], [H|T]), reverseList(T,[], S).

good([A]).
good([H|X]):- not(member(H,X)), good(X).

goodsbs(X,Y):- good(X), sms(X,Y).

%% goodsbs(X,X).
%% goodsbs([H|G],X):- member(H,X), not(member(H,G)), goodsbs(G,X).
%% goodsbs([],_).

%% goodsbs(U,U).
%% goodsbs(U,L) :- 
%%	length(L,Y), 
%%	         sbs(U,L),
  %%      length(U,N),
%% N < Y.

equallength(V,U):- length(V,N), length(U,M), M = N.

goodsbs2(X,X).
goodsbs2([X],Y):- member(X,Y).
goodsbs2([H|G],X):- member(H,X), not(member(H,G)), goodsbs2(G,X).
 
%%% subset(N, InList, Out) :-
 %%   splitSet(InList,_,SubList),
  %%  permutation(SubList,Out),
  %%  length(Out, N).

%%% splitSet([ ],[ ],[ ]).
%%% splitSet([H|T],[H|L],R) :-
   %%% splitSet(T,L,R).
%%% splitSet([H|T],L,[H|R]) :-
  %%%  splitSet(T,L,R).

