%%% Filename : ac_printcmd.pl
%%% Date     : 23.02.2011
%%% Contents : prints the rules on the command-line
%%% Author   : Lara Katharina Spendier <lara@logic.at>


%% printrule(+Rule, +Class)
%%			prints the rule on the commandline; if the rule is in the classes p(1), p(2) or
%%			p(3), it is a hypersequent
printrule([P, C], Class) :-
	member(Class, [p(1), p(2), p(3)]),
	nl, printpremises(P, Class,0),
	writef('\n-----------------------------\n G'),
	printconclusion(C, Class).

printrule([P, C], Class) :-
	\+ member(Class, [p(1), p(2), p(3)]),
	nl, printpremises(P, Class, 0),
	writef('\n-----------------------------\n'),
	printconclusion(C, Class).

%% printpremises(+Premise, +Class, +Counter)
%%			prints the premises of the rule
printpremises([], _,_) :-
	write(' ').
printpremises([P|T1], Class,2) :-
	write('\n'), printpremise(P, Class),
	printpremises(T1, Class,1).
printpremises([P|T1], Class,Counter) :-
	Counter1 is Counter+1,
	printpremise(P, Class),
	printpremises(T1, Class,Counter1).

%% printpremise(+Premise, +Class)
%%			prints one premise on the command line
printpremise([[], T], Class) :-
	member(Class, [p(1), p(2), p(3)]),
	writef('G|=> %p  ', [T]).
printpremise([[], T], Class) :-
	\+ member(Class, [p(1), p(2), p(3)]),
	writef(' => %p  ', [T]).
printpremise([H, T], Class) :-
	member(Class, [p(1), p(2), p(3)]),
	writef('G|'),
	printantecedent(H),
	writef('=> %p  ', [T]).
printpremise([H, T], Class) :-
	\+ member(Class, [p(1), p(2), p(3)]),
	printantecedent(H),
	writef('=> %p  ', [T]).

%% printconclusion(+Conclusion, Class)
%%			prints the conclusion of the rule
printconclusion([], _) :-
	write('  ').
printconclusion([[[], T]|T1], Class) :-
	member(Class, [p(1), p(2), p(3)]),
	writef('| => %p', [T]),
	printconclusion(T1, Class).
printconclusion([[H, T]|T1], Class) :-
	member(Class, [p(1), p(2), p(3)]),
	writef('| '),
	printantecedent(H),
	writef('=> %p', [T]),
	printconclusion(T1, Class).
printconclusion([[[], T]|T1], Class) :-
	\+ member(Class, [p(1), p(2), p(3)]),
	writef(' => %p', [T]),
	printconclusion(T1, Class).
printconclusion([[H, T]|T1], Class) :-
	\+ member(Class, [p(1), p(2), p(3)]),
	printantecedent(H),
	writef('=> %p ', [T]),
	printconclusion(T1, Class).

%% printantecedent(+Antecedent)
%%			prints the antecedent
printantecedent([]).
printantecedent([H|[]]) :-
	writef('%p ', [H]).
printantecedent([H|T]) :-
	writef('%p,', [H]),
	printantecedent(T).
