%%% Filename : ac_tex.pl
%%% Date     : 16.03.2012
%%% Contents : module for writing the output into a tex file
%%% Author   : Lara Katharina Spendier <lara@logic.at>

%% texOut(+Axiom, +Rules, +C)
%%	Axiom 	    ... Axiom the user provided
%% 	Rules  	    ... Analytic Rules for the axiom ([[Premises, Conclusion],...])
%%	C 	    ... Class the axiom is in
texOut(Axiom, Rules, C) :-
	member(C, [p(1), n(1), n(2)]),
	axiom2tex(Axiom,TAxiom,[]),
	tell('tex/AxiomCalc.sty'), 
	length(Rules, N),
	constants2tex("FLe", N, C),
	print_phrase(texNewAxiom(TAxiom)),
	print_phrase(texNewAxiomNotMath(TAxiom)),
	nl, print_phrase(texNewRuleStart),
	createSeqRules(Rules),
	print_phrase(texNewRuleEnd),
	told.

texOut(Axiom, Rules, C) :-
	member(C, [p(2), p(3)]),
	axiom2tex(Axiom,TAxiom,[]),
	tell('tex/AxiomCalc.sty'), 
	length(Rules, N),
	constants2tex("HFLew", N, C),
	print_phrase(texNewAxiom(TAxiom)),
	print_phrase(texNewAxiomNotMath(TAxiom)),
	nl, print_phrase(texNewRuleStart),
	createHSeqRules(Rules),
	print_phrase(texNewRuleEnd),
	told.

texOut(_,_,C) :-
	\+ member(C, [p(1),p(2),p(3),n(1),n(2)]),
	tell('tex/AxiomCalc.sty'),
	write('No'),
	told.

%% texOutSC(+Axiom, +Rules, +C)
%%	Axiom 	    ... Axiom the user provided
%% 	Rules  	    ... Analytic Rules for the axiom ([[Premises, Conclusion],...])
%%	C 	    ... Class the axiom is in

texOutSC(Axiom, Rules, C, 1, []) :-
	member(C, [p(1), n(1), n(2)]),
	axiom2tex(Axiom,TAxiom,[]),
	tell('tex/AxiomCalcSC.sty'), 
	length(Rules, N),
	constants2tex("MTL", N, C),
	print_phrase(texNewAxiom(TAxiom)),
	print_phrase(texNewAxiomNotMath(TAxiom)),
	print_phrase(texConvOrSA("convergent")),
	print_phrase(texConvOrSAText("ConvText")),
	print_phrase(texConvergentRule("The premise(s) of the rule satisfy condition (2) in Definition~6.")),
	print_phrase(texStandardComplete("SCC")),
	print_phrase(texAbstract("AbstractSC")),
	print_phrase(texTitle("TitleSC")),
	print_phrase(texSectionTitle("SectionTitleSC")),
	nl, print_phrase(texNewRuleStart),
	createHSeqRules(Rules),
	print_phrase(texNewRuleEnd),
	told.

%texOutSC(Axiom, Rules, C, 1, [_, Conv]) :-
%	\+ member(C, [p(1), n(1), n(2)]),
%	axiom2tex(Axiom,TAxiom,[]),
%	tell('tex/AxiomCalcSC.sty'), 
%	length(Rules, N),
%	constants2tex("MTL", N, C),
%	print_phrase(texNewAxiom(TAxiom)),
%	print_phrase(texNewAxiomNotMath(TAxiom)),
%	print_phrase(texConvOrSA("convergent")),
%	( (Conv = []) ->
 %   	  print_phrase(texConvergentRule("The premise(s) of the rule satisfy condition (1) in Definition~6."))
%	; reverseList(Conv, [], ConvR),
%	  print_phrase(texConvergentRule("The premise(s) of the rule satisfy conditions (1)-(3) in Definition~6. Indeed: \\begin{itemize}",pivot2tex(ConvR),"\\end{itemize}"))
%	),
%	print_phrase(texStandardComplete("SC")),
%	print_phrase(texAbstract("AbstractSC")),
%	print_phrase(texTitle("TitleSC")),
%	print_phrase(texSectionTitle("SectionTitleSC")),
%	nl, print_phrase(texNewRuleStart),
%	createHSeqRules(Rules),
%	print_phrase(texNewRuleEnd),
%	told.

%texOutSC(Axiom, Rules, C, 0, [Prem, _]) :-
%	axiom2tex(Axiom,TAxiom,[]),
%	tell('tex/AxiomCalcSC.sty'), 
%	length(Rules, N),
%	constants2tex("MTL", N, C),
%	print_phrase(texNewAxiom(TAxiom)),
%	print_phrase(texNewAxiomNotMath(TAxiom)),
%	print_phrase(texConvOrSA("convergent")),
%	print_phrase(texConvergentRule(prem2tex(Prem))),
%	print_phrase(texStandardComplete("SCN")),
%	print_phrase(texAbstract("AbstractSCN")),
%	print_phrase(texTitle("TitleSCN")),
%	print_phrase(texSectionTitle("SectionTitleSCN")),
%	nl, print_phrase(texNewRuleStart),
%	createHSeqRules(Rules),
%	print_phrase(texNewRuleEnd),
%	told.

texOutSC(_,_,_,100,_) :-
	tell('tex/AxiomCalcSC.sty'),
	write('No'),
	told.

texOutSCA(Axiom, Rules, C, Convergent, _, 1) :-
	\+ member(C, [p(1), n(1), n(2)]),
	axiom2tex(Axiom,TAxiom,[]),
	tell('tex/AxiomCalcSC.sty'), 
	length(Rules, N),
	constants2tex("MTL", N, C),
	print_phrase(texConvOrSA("semi-anchored")),
	print_phrase(texConvOrSAText("SAText")),
	print_phrase(texNewAxiom(TAxiom)),
	print_phrase(texNewAxiomNotMath(TAxiom)),
%	( (Conv = []) ->
%    	  print_phrase(texConvergentRule("The premise(s) of the rule satisfy condition (1) in Definition~6."))
%	; reverseList(Conv, [], ConvR),
%	  print_phrase(texConvergentRule("The premise(s) of the rule satisfy conditions (1)-(3) in Definition~6. Indeed: \\begin{itemize}",pivot2tex(ConvR),"\\end{itemize}"))
%	),
    ( (Convergent = 0) ->
    	print_phrase(texNotConvergent("not"));
    	print_phrase(texNotConvergent(" "))
    ),
	print_phrase(texStandardComplete("SCSA")),
	print_phrase(texAbstract("AbstractSC")),
	print_phrase(texTitle("TitleSC")),
	print_phrase(texSectionTitle("SectionTitleSC")),
	nl, print_phrase(texNewRuleStart),
	createHSeqRules(Rules),
	print_phrase(texNewRuleEnd),
	told.
	
texOutSCA(Axiom, Rules, C, Convergent, _, 0) :-
	\+ member(C, [p(1), n(1), n(2)]),
	axiom2tex(Axiom,TAxiom,[]),
	tell('tex/AxiomCalcSC.sty'), 
	length(Rules, N),
	constants2tex("MTL", N, C),
	print_phrase(texNewAxiom(TAxiom)),
	print_phrase(texNewAxiomNotMath(TAxiom)),
	print_phrase(texConvOrSA("semi-anchored")),
	print_phrase(texConvOrSAText("SAText")),
    ( (Convergent = 0) ->
    	print_phrase(texNotConvergent("not"));
    	print_phrase(texNotConvergent(" "))
    ),
	print_phrase(texStandardComplete("SCSAN")),
	print_phrase(texAbstract("AbstractSCN")),
	print_phrase(texTitle("TitleSCN")),
	print_phrase(texSectionTitle("SectionTitleSCN")),
	nl, print_phrase(texNewRuleStart),
	createHSeqRules(Rules),
	print_phrase(texNewRuleEnd),
	told.


%% constants2tex(+BaseCalculus)
%%	prints the style information according to BaseCalculus to the style file
constants2tex("FLe", N, _) :-
	( (N = 1) ->
	  print_phrase(texRule("rule"))
 	; print_phrase(texRule("rules"))
	),
	print_phrase(texAxiomClass("{N}_2")),
	print_phrase(texWrittenBC("sequent")),
	print_phrase(texNameBaseCalculus("FLe")),
	print_phrase(texNameBaseCalculusP("FLe")),
	print_phrase(texFullLambek("FullLambekSeq")),
	print_phrase(texBaseCalculus("CalculusFLe")),
	print_phrase(texEquivalences("EquivalenceSeq")),
	print_phrase(texTVPos("TVFLeP")),
	print_phrase(texTVNeg("TVFLeN")),
	print_phrase(texTVText("TVTexte")).

constants2tex("HFLew", N, C) :-
	( N = 1 ->
	  print_phrase(texRule("rule"))
 	; print_phrase(texRule("rules"))
	),
	( member(C, [p(2)]) ->
	  print_phrase(texAxiomClass("{P}_2"))
	; print_phrase(texAxiomClass("{P}_3"))
	),
	print_phrase(texWrittenBC("hypersequent")),
	print_phrase(texNameBaseCalculus("HFLew")),
	print_phrase(texNameBaseCalculusP("FLew")),
	print_phrase(texFullLambek("FullLambekHSeq")),
	print_phrase(texBaseCalculus("CalculusHFLew")),
	print_phrase(texEquivalences("EquivalenceHSeq")),
	print_phrase(texTVPos("TVFLewP")),
	print_phrase(texTVNeg("TVFLewN")),
	print_phrase(texTVText("TVTextw")).

constants2tex("MTL", N, C) :-
	( N = 1 ->
	  print_phrase(texRule("rule")),
	  print_phrase(texRuleVerb("is"))
 	; print_phrase(texRule("rules")),
	  print_phrase(texRuleVerb("are"))
	),
	( member(C, [p(2)]) ->
	  print_phrase(texAxiomClass("{P}_2"))
	; ( member(C, [n(2)]) ->
	    print_phrase(texAxiomClass("{N}_2"))
	  ; print_phrase(texAxiomClass("{P}_3"))
	  )
	).

%% createSeqRule(+[Rules])
%% 	[Rules]  ... Several analytic rules for the class N2
createSeqRules([]).
createSeqRules([[Prem, Con]|T]) :-
	createSeqRule(Prem, Con),
	createSeqRules(T).

%% createHSeqRule(+[Rules])
%% 	[Rules]  ... Several analytic hypersequent rules
createHSeqRules([]).
createHSeqRules([[Prem, Con]|T]) :-
	createHSeqRule(Prem, Con),
	createHSeqRules(T).

%% createSeqRule(+Prem, +Con)
%% 	Prem,Con  ... Analytic Sequent Rule for the axiom (Premises, Conclusion)
createSeqRule(Prem, Con) :-
	length(Prem,N),
	member(N, [0,1]),
	rule2tex(Prem, TPrem, []),
	rule2tex(Con, TCon, []),
	print_phrase(texNewAxC(TPrem)),
	print_phrase(texNewUnCon(TCon)).

createSeqRule(Prem, Con) :-
	length(Prem,N),
	member(N, [2,4]),
	createPremises(Prem, TPrem),
	N1 is N/2+1,
	printBinPremises(0, TPrem, N1, 1),
	rule2tex(Con, TCon, []),
	print_phrase(texNewBinCon(TCon)).

createSeqRule(Prem, Con) :-
	length(Prem,N),
	createPremises(Prem, TPrem),
	N1 is N//3,
	N2 is N mod 3,
	printTriPremises(0, TPrem, N1, N2, 1),
	rule2tex(Con, TCon, []),
	print_phrase(texNewTriCon(TCon)).

%% createHSeqRule(+Prem, +Con)
%% 	Prem,Con  ... Analytic Hypersequent Rule for the axiom (Premises, Conclusion)
createHSeqRule(Prem, Con) :-
	length(Prem,N),
	member(N, [0,1]),
	rule2tex(Prem, TPrem1, []),
	append("G\\hh",TPrem1,TPrem),
	createHSeqConclusion(Con, TCon1),
	append("G",TCon1,TCon),
	nl,
%	print_phrase(texNewRuleStart),
	print_phrase(texNewAxC(TPrem)),
	print_phrase(texNewUnCon(TCon)).
%	print_phrase(texNewRuleEnd).

createHSeqRule(Prem, Con) :-
	length(Prem,N),
	member(N, [2,4]),
	createPremises(Prem, TPrem),
	nl,
%	print_phrase(texNewRuleStart),
	N1 is N/2+1,
	printBinPremises(1, TPrem, N1, 1),
	createHSeqConclusion(Con, TCon1),
	append("G",TCon1,TCon),
	print_phrase(texNewBinCon(TCon)).
%	print_phrase(texNewRuleEnd).

createHSeqRule(Prem, Con) :-
	length(Prem,N),
	createPremises(Prem, TPrem),
	nl,
%	print_phrase(texNewRuleStart),
	N1 is N//3,
	N2 is N mod 3,
	printTriPremises(1, TPrem, N1, N2, 1),
	createHSeqConclusion(Con, TCon1),
	append("G",TCon1,TCon),
	print_phrase(texNewTriCon(TCon)).
%	print_phrase(texNewRuleEnd).

%% createHSeqConclusion(+Con,-TCon])
%% 	Con  ... Conclusion of the rule computed by AxiomCalc
%%	TCon ... Conclusion of the rule in tex-format
createHSeqConclusion([], []).
createHSeqConclusion([H|T], TCon) :-
	rule2tex(H, TH, []),
	createHSeqConclusion(T, TCon1),
	append(TH, TCon1, TCon2),
	append("\\hh",TCon2,TCon).

%% createPremises(+Prem,-TPrem])
%% 	Prem  ... Premises of the rule computed by AxiomCalc
%%	TPrem ... Premises of the rule in tex-format
createPremises([], []).
createPremises([H|T], TPrem) :-
	rule2tex(H, TH, []),
	createPremises(T, TPrem1),
	append([TH],TPrem1,TPrem).

%% printBinPremises(+Hyper, +Prem, +N, +I])
%%	for (hyper)sequent rules that have 2 or 4 premises
%% 	Hyper ... is 1 if hypersequent, 0 if sequent
%%	Prem  ... Premises of the rule
%%	N     ... number of premises
%%	I     ... counter
printBinPremises(_, [], _, _).

printBinPremises(Hyper, [H|T], N, I) :-
	member(I, [1,N]),
	I1 is I+1,
	( Hyper = 1 ->
	  append("G\\hh",H,H1)
	; H1 = H
	),
	print_phrase(texNewAxC(H1)),
	printBinPremises(Hyper, T, N, I1).
	
printBinPremises(Hyper, [H|T], N, I) :-
	I1 is I+1,
	( Hyper = 1 ->
	  append("G\\hh",H,H1)
	; H1 = H
	),
	print_phrase(texNoLine),
	print_phrase(texNewUnC(H1)),
	printBinPremises(Hyper, T, N, I1).

%% printTriPremises(+Hyper, +Prem, +Div, +Mod, +I])
%%	for (hyper)sequent rules that have 3 or >4 premises
%% 	Hyper ... is 1 if hypersequent, 0 if sequent
%%	Prem  ... Premises of the rule
%%	Div   ... number of premises / 3 (Integer division)
%%	Mod   ... number of premises mod 3
%%		  --> to balance the number of premises in the rule!
%%	I     ... counter
printTriPremises(_, [], _, _, _).

printTriPremises(Hyper, [H|T], Div, Mod, I) :-
	( Mod > 0 ->
 	  M1 is 1+Div+1
	; M1 is 1+Div
	), 
	( Mod > 1 ->
	  M2 is M1+Div+1
	; M2 is M1+Div
	),
	member(I, [1,M1,M2]),
	I1 is I+1,
	( Hyper = 1 ->
	  append("G\\hh",H,H1)
	; H1 = H
	),
	print_phrase(texNewAxC(H1)),
	printTriPremises(Hyper, T, Div, Mod, I1).
	
printTriPremises(Hyper, [H|T], Div, Mod, I) :-
	I1 is I+1,
	( Hyper = 1 ->
	  append("G\\hh",H,H1)
	; H1 = H
	),
	print_phrase(texNoLine),
	print_phrase(texNewUnC(H1)),
	printTriPremises(Hyper, T, Div, Mod, I1).
